/* -*-  vim: set ts=2 nowrap: -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy
 * of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an
 * "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is PubMed toolbar code.
 *
 * The Initial Developer of the Original Code is Adrian
 * Mastronardi <adrianm@museo.fcnym.unlp.edu.ar>. Portions
 * created by Adrian Mastronardi are Copyright (C) 2002 Adrian
 * Mastronardi. All Rights Reserved.
 *
 * Contributor(s): 
 *   Martin Hassman <martin.hassman@vscht.cz>
 *
 * $Id: expasybar.js,v 1.16 2002/10/19 19:30:32 Martin Hassman Exp $
 */

function expasybar_SearchObject(search_where) {
	/* obtain data and constructing template request */
	/*
		Object properties : 
			main_url
			search_url		requesting url (without request data)
			search_param	template for requesting data
			method				GET | POST
			label
			type					"simple" | "complex"
	 */

  this.label = search_where;      // don't used now, label shown in combobox
	this.method = "GET";	          // only GET method working now
	this.type = "simple";           // if type = "simple" there is only simple changing __WORD__ to search term

	// TODO: allow user change this data and saved it under user preferences
	switch (search_where) {
	case "swissprot_description":
		this.main_url = expasybar_getserver() + "cgi-bin/sprot-search-de";
		this.search_url = expasybar_getserver() + "cgi-bin/sprot-search-de"	
		this.search_param = "SEARCH=__WORD__&S=1&T=1";
		break;

	case "swissprot_full":
		this.main_url = expasybar_getserver() + "cgi-bin/sprot-search-ful";
		this.search_url = expasybar_getserver() + "cgi-bin/sprot-search-ful";
		this.search_param = "SEARCH=__WORD__&S=on&T=on";
		break;

	case "prosite_full":
		this.main_url = expasybar_getserver() + "cgi-bin/prosite-search-ful";
		this.search_url = expasybar_getserver() + "cgi-bin/prosite-search-ful";
		this.search_param = "SEARCH=__WORD__";
		break;

	case "swiss2dpage":
		this.main_url = expasybar_getserver() + "cgi-bin/ch2d-search-ful";
		this.search_url = expasybar_getserver() + "cgi-bin/ch2d-search-ful";
		this.search_param = "SEARCH=__WORD__";
		break;

	case "enzyme_name":
		this.main_url = expasybar_getserver() +"enzyme/";
		this.search_url = expasybar_getserver() + "cgi-bin/enzyme-search-de"	// doplnovat otazniky!!!
		this.search_param = "query=__WORD__";
		break;

	case "enzyme_number":
		this.main_url = expasybar_getserver() + "enzyme/";
		this.search_url = expasybar_getserver() + "cgi-bin/enzyme-search-ec";
		this.search_param = "field1=__1__&field2=__2__&field3=__3__&field4=__4__";
		this.type = "complex";
		break;
				
	case "enzyme_cofactor":
		this.main_url = expasybar_getserver() + "cgi-bin/enzyme-search-cf";
		this.search_url = expasybar_getserver() + "cgi-bin/enzyme-search-cf"	
		this.search_param = "__WORD__";
		break;

	case "enzyme_class":
		this.main_url = expasybar_getserver() + "cgi-bin/enzyme-search-cl";
		this.search_url = expasybar_getserver() + "cgi-bin/get-enzyme-entry-unprecise"	
		this.search_param = "__WORD__";
		break;

	case "enzyme_comments":
		this.main_url = expasybar_getserver() + "cgi-bin/enzyme-search-cc";
		this.search_url = expasybar_getserver() + "cgi-bin/enzyme-search-cc"	
		this.search_param = "__WORD__";
		break;

  default:
		// only as signal for bug :-)
		this.main_url = "http://www.google.com/";
		this.search_url = "http://www.google.com/search";
		this.search_param = "q=__WORD__";
	}
	
}


function expasybar_RequestObject(search, query) {
	/* object constructing request from template in search object */
	/* Object properties:
			method      "GET" | "POST"  - only GET is supported now!
			url         requesting url
			postdata    preparation for futere supporting of POST :-)
	 */

	this.postdata = "";
	this.method = search.method;

	if (search.type == "simple") {
		var re = /(.*)(__WORD__)(.*)/;
		this.url = search.search_url + "?" + search.search_param.replace(re, "$1" + query + "$3");
		return;
	}

	// COMPLEX SEARCH
	switch(search.label) {

	case "enzyme_number" :
		// extract four number from query
		var ec = query.split(".");
		var re = /(.*)(__1__)(.*)(__2__)(.*)(__3__)(.*)(__4__)(.*)/;

		this.url = search.search_url + "?" + search.search_param.replace(re, "$1" + ec[0] + "$3" + ec[1] + "$5" + ec[2] + "$7" + ec[3] + "$9");
		break;

	default:
		alert('DEBUG - error in expasybar_RequestObject() unknown search.label');
	}

}


function expasybar_Go(winopen) {
	/* start searching */

	var query = document.getElementById("expasybar_query").value;
	var search_where = document.getElementById("expasybar_selectwhere").value; 	// server where to search
	var search = new expasybar_SearchObject(search_where);

	if(!query) { 
		expasybar_LaunchSearch(search.main_url, winopen);
		return; 
	}

	var request = new expasybar_RequestObject(search, query);

	// added new item to history
	expasy_AddToolbarHistory(query);

	// for debug
	//alert(request.url);
	expasybar_LaunchSearch(request.url, winopen);
}


function expasybar_LaunchSearch(url, winopen) {
	/* Launch search - there are multiple way (normal/new tab/window) */

	switch(winopen) {
	case 0 :	// launch search in this window
		_content.location.href = url;
		break;
		
	case 1 :	// launch search in new tab
		var browser = getBrowser();
		browser.addTab(url);
		break;
		
	case 2 :	// launch search in new window
		open(url);
		break;

	default :
		_content.location.href = url;
	}
}


function expasy_AddToolbarHistory(query) {
	// add item to history of search queries
	var myMenuItem = document.createElement("menuitem");
	var queryBox = document.getElementById("expasybar_query");

	myMenuItem.setAttribute("label", query);
	queryBox.childNodes.item(0).appendChild(myMenuItem);
}


function expasybar_HandleKeyEvent(inEvent) {
	// original from by EasySearch project developers
	// in which window dislay result
	var winopen = 0;	// default 0 = this window

	if(inEvent.keyCode != 13)
		return;

	// if ctrl set = 1 = in tab
	if(inEvent.ctrlKey)
		winopen = 1;

	// if shift set = 2 = in new window
	if(inEvent.shiftKey)
		winopen = 2;

	expasybar_Go(winopen);
}


function expasybar_FindText() {
	/* find text in browser window */
	expasybar_FindInPage(document.getElementById("expasybar_query").value);
}


function expasybar_FindInPage(sQuery) {
	// started with copy of code from chrome://communicator/content/findUtils.js findAgainInPage function
	// FIXME now is there error - don't worhing CTRL+G (Find Again) with this text

	var findInst = getBrowser().webBrowserFind;
	var findInFrames = findInst.QueryInterface(Components.interfaces.nsIWebBrowserFindInFrames);

	findInFrames.rootSearchFrame = window._content;

	var focusedWindow = document.commandDispatcher.focusedWindow;
	if (!focusedWindow || focusedWindow == window)
		focusedWindow = window._content;
	
	findInFrames.currentSearchFrame = focusedWindow;
	findInst.searchFrames = true;

	// get the find service, which stores global find state, and init the
	// nsIWebBrowser find with it. We don't assume that there was a previous
	// Find that set this up.
	var findService = Components.classes["@mozilla.org/find/find_service;1"].getService(Components.interfaces.nsIFindService);

	findInst.searchString  = sQuery;
	findInst.matchCase     = findService.matchCase;
	findInst.wrapFind      = findService.wrapFind;
	findInst.entireWord    = findService.entireWord;
	findInst.findBackwards = findService.findBackwards;

	var found = false;
	if (findInst.searchString.length == 0) {
		// no previous find text - start standard find function witch display find dialog
		findInPage(getBrowser(), window._content, focusedWindow);
		// copy text to editbox FIXME - do not worked good
		document.getElementById("expasybar_query").value = findService.searchString;
		return;
	}

	found = findInst.findNext();

	if (!found) {
		// Show "not found" dialog
		var PromptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService().QueryInterface(Components.interfaces.nsIPromptService); 
		var FindBundle = document.getElementById("findBundle");
		PromptService.alert(window, FindBundle.getString("notFoundTitle"), FindBundle.getString("notFoundWarning"));
	}      
}


function expasybar_LoadFindParameters() {
	// load actual parameters of Find in Page and check corresponding menu items
	// get the find service, which stores global find state
	var findService = Components.classes["@mozilla.org/find/find_service;1"].getService(Components.interfaces.nsIFindService);

	if (findService.matchCase == true)
		document.getElementById("expasybar-findtextmenu-matchcase").setAttribute("checked", findService.matchCase);
	else
		document.getElementById("expasybar-findtextmenu-matchcase").removeAttribute("checked");	
	
	if (findService.wrapFind == true)
		document.getElementById("expasybar-findtextmenu-wraparound").setAttribute("checked", findService.wrapFind);
	else
		document.getElementById("expasybar-findtextmenu-wraparound").removeAttribute("checked");	

	if (findService.findBackwards == true)
		document.getElementById("expasybar-findtextmenu-backwards").setAttribute("checked", findService.findBackwards);
	else
		document.getElementById("expasybar-findtextmenu-backwards").removeAttribute("checked");	
}


function expasybar_SaveFindParameters() {
	// save corresponding find parameters from checked menu items	to mozilla find service
	// BUG if are parameters often changed without searching sometimes are not saved values showed in find dialog
	// I do not known why, after using find are synchronized

	var boolMatchCase	= document.getElementById("expasybar-findtextmenu-matchcase").getAttributeNode("checked")  != null ? true : false;
	var boolWrapFind = document.getElementById("expasybar-findtextmenu-wraparound").getAttributeNode("checked") != null ? true : false;
	var boolFindBackwards	= document.getElementById("expasybar-findtextmenu-backwards").getAttributeNode("checked") != null ? true : false;
	
	// get the find service, which stores global find state
	var findService = Components.classes["@mozilla.org/find/find_service;1"].getService(Components.interfaces.nsIFindService);
	
	findService.matchCase = boolMatchCase;
	findService.wrapFind = boolWrapFind;
	findService.findBackwards = boolFindBackwards;
}


function expasybar_HighlightText() {
	// highlight query text on page
	var page = document.getElementById("content").contentDocument;

	if (document.getElementById("expasybar_query").value == "") {
		alert("No text for highlighting");
		return;
	}
	
	//alert(document.getElementById("expasybar-highlightcolorpicker").color);	
	expasybar_HighlightInFrames(page);
}


function expasybar_HighlightInFrames(page) {
	// search in all (also nested) frames and iframes - rewritten from bannerblind mozilla project
	var frame, iframe;

	if (page.getElementsByTagName("frame").length > 0) {
		for (var f = 0; f < page.getElementsByTagName("frame").length; f++) {
			frame = page.getElementsByTagName("frame").item(f).contentDocument;
			expasybar_HighlightInFrames(frame);
		}
		return;
	}
	
	expasybar_HighlightInPage(page);

	if (page.getElementsByTagName("iframe").length > 0) {
		for (var i = 0; i < page.getElementsByTagName("iframe").length; i++) {
			iframe = page.getElementsByTagName("iframe").item(i).contentDocument;
			expasybar_HighlightInFrames(iframe);
		}	
	}
}


function expasybar_HighlightInPage(page) {
	// highlight text in page
	var sText = document.getElementById("expasybar_query").value.toUpperCase();
	var textlength = sText.length;
	var dview = page.defaultView;
	var highlight_color = document.getElementById("expasybar-highlightcolorpicker").color;

	if (!highlight_color)
		highlight_color = "yellow";

	expasybar_searchWithinNode(page.body);
	
	function expasybar_searchWithinNode(node) {
		// original get from Jesse Bookmarklets page
		var pos, spannode, middlebit, endbit, middleclone;

		if (node.nodeType == 3) {	// 3 == TEXT_NODE
			pos = node.data.toUpperCase().indexOf(sText);  // find text position

			if (pos < 0)
				return 0;

			// text was found, add <span> element with background color
			spannode = page.createElement("span");
			spannode.style.backgroundColor = highlight_color;
			middlebit = node.splitText(pos);	// breaks node before sText
			endbit = middlebit.splitText(textlength);	// breaks node after sText
			middleclone = middlebit.cloneNode(true);	// copy node with (only) sText
			spannode.appendChild(middleclone);				// and add it in <span>
			middlebit.parentNode.replaceChild(spannode, middlebit);	// replace old sText node with <span>
			return 1;
		}

		// nodeType 1 == ELEMENT_NODE, has child, is displayed
		if (node.nodeType == 1 && node.childNodes && dview.getComputedStyle(node, null).getPropertyValue("display") != "none") {
				var child = 0;
				while (child < node.childNodes.length) {
					child += expasybar_searchWithinNode(node.childNodes[child]); // recursive calling, child correction to new created <span>
					child++;
				}
		}
		return 0;
	}	// function expasybar_searchWithinNode(node)
}


function expasybar_getserver() {
	// obtain server name from mozilla preferences and return it as usefull string e.g. http://www.expasy.org
	var pref = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPref);
	var s;

	s = "http://" + pref.GetCharPref("expasybar.mirror.name") + "/";
	return (s);
}


function expasybar_getftpserver() {
	// obtain server name from mozilla preferences and return it as usefull string e.g. ftp://www.expasy.org
	var pref = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPref);
	var s;

	s = "ftp://" + pref.GetCharPref("expasybar.mirror.name") + "/";
	return (s);
}

function expasybar_CheckPreferences() {
	// Check wheter there are created all expasybar preferences, if not, set all to default values
	var pref = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPref);

	try {
		pref.GetCharPref("expasybar.mirror.region");
		pref.GetCharPref("expasybar.mirror.name");
	}
	catch(e) {
		expasybar_SetDefaultPref();
	}
}


function expasybar_SetDefaultPref() {
	// set expasybar preferences to default values
	var pref = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPref);

	pref.SetCharPref("expasybar.mirror.region", "default");
	pref.SetCharPref("expasybar.mirror.name", "www.expasy.org");
}


/*************************************************\
|***********    Start of Init code     ***********|
\*************************************************/

expasybar_CheckPreferences();

/*************************************************\
|************    End of Init code     ************|
\*************************************************/

///////////////////////////
// DOCUMENT managing test
function expasybar_manage() {
	// only test for document manipulating

//	both method for document worked
//	var page = window._content.document;
	var page = document.getElementById("content").contentDocument;
	
	//page.body.background ="http://martin/mraky.jpg";
	//page.body.bgColor ="red";
	page.close();
	page.write("<html><head></head><body>Text</body></html>");
	page.close();
	window._content.location.href="http://martin";
}

// here may be loading data for every new window opening with help of addevent listener (like e.g. mozdevbar)
